from cpl_core.database.context import DatabaseContextABC
from cpl_discord.service import DiscordBotServiceABC

from bot_core.abc.client_utils_abc import ClientUtilsABC
from bot_core.abc.permission_service_abc import PermissionServiceABC
from bot_data.abc.achievement_repository_abc import AchievementRepositoryABC
from bot_data.abc.user_joined_game_server_repository_abc import (
    UserJoinedGameServerRepositoryABC,
)
from bot_data.abc.user_joined_server_repository_abc import UserJoinedServerRepositoryABC
from bot_data.abc.user_joined_voice_channel_repository_abc import (
    UserJoinedVoiceChannelRepositoryABC,
)
from bot_data.abc.user_warnings_repository_abc import UserWarningsRepositoryABC
from bot_data.model.user_history import UserHistory
from bot_graphql.abc.data_query_with_history_abc import DataQueryWithHistoryABC
from bot_graphql.filter.achievement_filter import AchievementFilter
from bot_graphql.filter.user_joined_game_server_filter import UserJoinedGameServerFilter
from bot_graphql.filter.user_joined_server_filter import UserJoinedServerFilter
from bot_graphql.filter.user_joined_voice_channel_filter import (
    UserJoinedVoiceChannelFilter,
)
from bot_graphql.filter.user_warning_filter import UserWarningFilter
from modules.level.service.level_service import LevelService


class UserQuery(DataQueryWithHistoryABC):
    def __init__(
        self,
        bot: DiscordBotServiceABC,
        db: DatabaseContextABC,
        levels: LevelService,
        client_utils: ClientUtilsABC,
        ujs: UserJoinedServerRepositoryABC,
        ujvs: UserJoinedVoiceChannelRepositoryABC,
        user_joined_game_server: UserJoinedGameServerRepositoryABC,
        permissions: PermissionServiceABC,
        achievements: AchievementRepositoryABC,
        user_warnings: UserWarningsRepositoryABC,
    ):
        DataQueryWithHistoryABC.__init__(self, "User", "UsersHistory", UserHistory, db)

        self._bot = bot
        self._levels = levels
        self._client_utils = client_utils
        self._user_joined_game_server = user_joined_game_server
        self._ujs = ujs
        self._ujvs = ujvs
        self._permissions = permissions
        self._achievements = achievements

        self.set_field("id", lambda user, *_: user.id)
        self.set_field("discordId", lambda user, *_: user.discord_id)
        self.set_field("name", lambda user, *_: user.name)
        self.set_field("xp", lambda user, *_: user.xp)
        self.set_field("messageCount", lambda x, *_: x.message_count)
        self.set_field("reactionCount", lambda x, *_: x.reaction_count)
        self.set_field("birthday", lambda x, *_: None if x.birthday is None else x.birthday.strftime("%d.%m.%Y"))
        self.set_field("ontime", lambda user, *_: user.ontime)
        self.set_field("gameOntime", lambda user, *_: user.game_ontime)
        self.set_field("level", lambda user, *_: user.level)
        self.set_field("activityScore", lambda user, *_: user.activity)
        self.add_collection(
            "joinedServer",
            lambda user, *_: self._ujs.get_user_joined_servers_by_user_id(user.id),
            UserJoinedServerFilter,
        )
        self.add_collection(
            "joinedVoiceChannel",
            lambda user, *_: self._ujvs.get_user_joined_voice_channels_by_user_id(user.id),
            UserJoinedVoiceChannelFilter,
        )
        self.add_collection(
            "userJoinedGameServer",
            lambda user, *_: self._user_joined_game_server.get_user_joined_game_servers_by_user_id(user.id),
            UserJoinedGameServerFilter,
        )
        self.add_collection(
            "achievement",
            lambda user, *_: achievements.get_achievements_by_user_id(user.id),
            AchievementFilter,
        )
        self.add_collection(
            "userWarning",
            lambda user, *_: user_warnings.get_user_warnings_by_user_id(user.id),
            UserWarningFilter,
        )

        self.set_field("server", lambda user, *_: user.server)
        self.set_field("leftServer", lambda user, *_: user.left_server)
